import re

from pygments.lexer import RegexLexer, include, bygroups, using, this
from pygments.style import Style
from pygments.token import Error, Punctuation, Text, Comment, Operator, Keyword, Name, String, Number

class PseudocodefrStyle(Style):
    """Custom Style for Pseudocode fr"""
    default_style = ""
    styles = {
        Comment:                'italic #888',
        Keyword:                'bold #005',
        Name:                   '#f00',
        Name.Function:          '#0f0',
        Name.Class:             'bold #0f0',
        String:                 'bg:#eee #111'
    }

class PseudocodefrLexer(RegexLexer):
    '''
    A Pseudo code (fr) lexer
    '''
    name = 'Pseudocode'
    aliases = ['pseudocode', 'pseudo', 'algorithm', 'algo']
    filenames = ['*.algo', '*.pseudocode']
    mimetypes = []
    flags = re.IGNORECASE

    def op_replace(lexer, match):
        op = match.group(0)

        S = ('<=', '>=', '<>', '!=', '<-', '->', '^')
        R = ('≤',  '≥',  '≠',  '≠', '←', '→', '↑')

        if op in S:
            op = R[S.index(op)]

        yield match.start(), Operator, op

    def scomment(lexer, match):
        s = match.group(1).lower().strip()
        c = Comment

        directives = ['passage par copie', 'passage par valeur', 'passage par référence', 'passage par reference', 'passage par adresse', 've', 'vs', 've/s']

        if s in directives:
            c = Comment.Special

        yield match.start(), c, match.group(0)

    tokens = {
        'root': [
                 (r'\/\*.*\*\/', Comment),
                 (r'(\/\/|#).*\n', Comment),
                 (r'\|', Comment),
                 (r'\{(.*)\}', scomment),
                 include('strings'),
                 include('core'),
                 (r'[a-zéàèùçâêîôûÉÀÈÒÙÇÂÊÎÔÛ][a-z0-9éàèùçâêîôûÉÀÈÙÒÇÂÊÎÔÛ_]*', Name.Variable),
                 include('nums'),
                 (r'[\s]+', Text)
        ],
        'core':[ # Statements
                 (r'\b(debut|début|fin|si|alors|sinon|fin[_ ]si|tant[ _]que|tantque|fin[ _]tantque|faire|répéter'
                  r'repeter|type|structure|fin[ _]structure|fonction|procédure|procedure|retourner|renvoyer|'
                  r'pour|fin[ _]pour|à|déclarations?|jusque|spécialise|specialise|comporte|super|public|privé|protégé|'
                  r'afficher|saisir|'
                  r'classe'
                  r')\s*\b', Keyword),

                 # Data Types
                 (r'\b(entiers?|chaines?|chaînes?|réels?|reels?|caractères?|caracteres?|booléens?|'
                  r'booleens?|tableaux?|rien)\s*\b', 
                  Keyword.Type),

                  (r'\b(vrai|faux|nil)\s*\b',
                   Name.Constant),
                  
                 # Operators
                 (r'(<=|>=|<>|!=|<-|\^|\*|\+|-|\/|<|>|=|\\\\|mod|←|↑|→|≤|≥|≠|÷|×|\.\.|\[|\]|\.|non|xou|et|ou)',
                  op_replace),

                 (r'(\(|\)|\,|\;|:)',
                  Punctuation),

                 #(r'\b(\[(VE|VS|VE/S)\])\s*\b',
                 # Keyword.Declaration),

                  # Intrinsics
                 (r'\b(sqrt|pow|cos|sin|tan|arccos|arcsin|arctan|arctan2|lire|ecrire|écrire|'
                  r'exp|ln|log|détruire|detruire'
                  r')\s*\b', Name.Builtin)
                ],

        'strings': [
                 (r'"([^"])*"', String.Double),
                 (r"'([^'])*'", String.Single),
                ],

        'nums': [
                 (r'\d+(?![.Ee])', Number.Integer),
                 (r'[+-]?\d*\.\d+([eE][-+]?\d+)?', Number.Float),
                 (r'[+-]?\d+\.\d*([eE][-+]?\d+)?', Number.Float)
                ],
        }




from setuptools import setup

VERSION = '0.1.3'

setup(
    name="pygments-lexer-pseudocode-fr",
    packages=["pygments_lexer_pseudocode_fr"],
    version=VERSION,
    install_requires=[ "pygments" ],
    keywords='pygments lexer syntax highlight pseudo pseudocode algorithme fr',
    entry_points="""[pygments.lexers]
    pseudocodefr = pygments_lexer_pseudocode_fr:PseudocodefrLexer
    [pygments.styles]
    pseudocodefrstyle = pygments_lexer_pseudocode_fr:PseudocodefrStyle
    """,
    author="Rodrigo Schwencke",
    author_email="rod2ik.dev@gmail.com",
    description="Pygments Lexer for French Pseudocode",
    long_description_content_type="text/markdown",
    long_description="""Project Page : [rod2ik/pygments-lexer-pseudocode-fr](https://gitlab.com/rod2ik/pygments-lexer-pseudocode-fr)

Some examples in these pages:

* MkHack3rs : https://eskool.gitlab.io/mkhack3rs/pseudocode/

This project is one of others mkdocs-related projects.  
Please have a look at this page for a more complete view of all projects compatible with for mkdocs:

* https://eskool.gitlab.io/mkhack3rs/

This project was initially a continuation of the great job of (from newer to older) :

* All newer Credits: [rod2ik/pygments-lexer-pseudocode-fr](https://gitlab.com/rod2ik/pygments-lexer-pseudocode-fr)
* Simon Watcher [svvac/pygments-lexer-pseudocode](https://github.com/svvac/pygments-lexer-pseudocode)
    
In order to get it work with pip, and ultimately with mkdocs and mkdocs-material.

Licences:

* All newer parts (Rodrigo Schwencke) are [GPLv3+](https://opensource.org/licenses/GPL-3.0)
* Older parts (Simon Watcher) are [MIT License](http://www.opensource.org/licenses/mit-license.php)""",
    url="https://gitlab.com/rod2ik/pygments-lexer-pseudocode-fr.git",
    license="GPLv3+",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Documentation',
        "Natural Language :: French",
        "Topic :: Text Processing",
        "Topic :: Utilities",
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows 10',

    ]
)
